@@ -42,6 +42,7 @@ e.g.
 """
 
 TRADES_AGG = OrderedDict((
+    ('Ticker', 'first'),
     ('Size', 'sum'),
     ('EntryBar', 'first'),
     ('ExitBar', 'last'),
@@ -194,7 +195,7 @@ def compute_stats(
         equity[:] = stats._equity_curve.Equity.iloc[0]
         for t in trades.itertuples(index=False):
             equity.iloc[t.EntryBar:] += t.PnL
-    return _compute_stats(trades=trades, equity=equity, ohlc_data=data,
+    return _compute_stats(orders=stats._orders, trades=trades, equity=equity, ohlc_data=data,
                           risk_free_rate=risk_free_rate, strategy_instance=stats._strategy)
 
 
@@ -404,18 +405,14 @@ class SignalStrategy(Strategy):
         """
         if isinstance(entry_size, pd.DataFrame) and len(entry_size.columns) == 1:
             entry_size = entry_size.iloc[:, 0]
-        if not isinstance(entry_size, pd.Series):
-            entry_size = pd.Series(entry_size, dtype=float)
-        entry_size = entry_size.replace(0, np.nan)
+        entry_size = pd.Series(entry_size, dtype=float).replace(0, np.nan)
         self.__entry_signal = self.I(entry_size, name='entry size', plot=plot,
                                      overlay=False, scatter=True, color='black')
 
         if exit_portion is not None:
             if isinstance(exit_portion, pd.DataFrame) and len(exit_portion.columns) == 1:
                 exit_portion = exit_portion.iloc[:, 0]
-            if not isinstance(exit_portion, pd.Series):
-                exit_portion = pd.Series(exit_portion, dtype=float)
-            exit_portion = exit_portion.replace(0, np.nan)
+            exit_portion = pd.Series(exit_portion, dtype=float).replace(0, np.nan)
             self.__exit_signal = self.I(exit_portion, name='exit portion', plot=plot,
                                         overlay=False, scatter=True, color='black')
 
